#include "jpeglib.h"

/**************** RGB -> YCbCr conversion: most common case **************/

/*
 * YCbCr is defined per CCIR 601-1, except that Cb and Cr are
 * normalized to the range 0..MAXJSAMPLE rather than -0.5 .. 0.5.
 * The conversion equations to be implemented are therefore
 *	Y  =  0.29900 * R + 0.58700 * G + 0.11400 * B
 *	Cb = -0.16874 * R - 0.33126 * G + 0.50000 * B  + CENTERJSAMPLE
 *	Cr =  0.50000 * R - 0.41869 * G - 0.08131 * B  + CENTERJSAMPLE
 * (These numbers are derived from TIFF 6.0 section 21, dated 3-June-92.)
 * Note: older versions of the IJG code used a zero offset of MAXJSAMPLE/2,
 * rather than CENTERJSAMPLE, for Cb and Cr.  This gave equal positive and
 * negative swings for Cb/Cr, but meant that grayscale values (Cb=Cr=0)
 * were not represented exactly.  Now we sacrifice exact representation of
 * maximum red and maximum blue in order to get exact grayscales.
 *
 * To avoid floating-point arithmetic, we represent the fractional constants
 * as integers scaled up by 2^16 (about 4 digits precision); we have to divide
 * the products by 2^16, with appropriate rounding, to get the correct answer.
 *
 * For even more speed, we avoid doing any multiplications in the inner loop
 * by precalculating the constants times R,G,B for all possible values.
 * For 8-bit JSAMPLEs this is very reasonable (only 256 entries per table);
 * for 12-bit samples it is still acceptable.  It's not very reasonable for
 * 16-bit samples, but if you want lossless storage you shouldn't be changing
 * colorspace anyway.
 * The CENTERJSAMPLE offsets and the rounding fudge-factor of 0.5 are included
 * in the tables to save adding them separately in the inner loop.
 */

#define SCALEBITS	16	/* speediest right-shift on some machines */
#define CBCR_OFFSET	((INT32) CENTERJSAMPLE << SCALEBITS)
#define ONE_HALF	((INT32) 1 << (SCALEBITS-1))
#define FIX2(x)		((INT32) ((x) * (1L<<SCALEBITS) + 0.5))

/* We allocate one big table and divide it up into eight parts, instead of
 * doing eight alloc_small requests.  This lets us use a single table base
 * address, which can be held in a register in the inner loops on many
 * machines (more than can hold all eight addresses, anyway).
 */

#define R_Y_OFF		0			/* offset to R => Y section */
#define G_Y_OFF		(1*(MAXJSAMPLE+1))	/* offset to G => Y section */
#define B_Y_OFF		(2*(MAXJSAMPLE+1))	/* etc. */
#define R_CB_OFF	(3*(MAXJSAMPLE+1))
#define G_CB_OFF	(4*(MAXJSAMPLE+1))
#define B_CB_OFF	(5*(MAXJSAMPLE+1))
#define R_CR_OFF	B_CB_OFF		/* B=>Cb, R=>Cr are the same */
#define G_CR_OFF	(6*(MAXJSAMPLE+1))
#define B_CR_OFF	(7*(MAXJSAMPLE+1))
#define TABLE_SIZE	(8*(MAXJSAMPLE+1))


/*
 * Initialize for RGB->YCC colorspace conversion.
 */

INT32 rgb_ycc_tab[TABLE_SIZE];

void rgb_ycc_start()
{

  INT32 i;

  /* Allocate and fill in the conversion tables. */
  

  for (i = 0; i <= MAXJSAMPLE; i++) {
    rgb_ycc_tab[i+R_Y_OFF] = FIX2(0.29900) * i;
    rgb_ycc_tab[i+G_Y_OFF] = FIX2(0.58700) * i;
    rgb_ycc_tab[i+B_Y_OFF] = FIX2(0.11400) * i     + ONE_HALF;
    rgb_ycc_tab[i+R_CB_OFF] = (-FIX2(0.16874)) * i;
    rgb_ycc_tab[i+G_CB_OFF] = (-FIX2(0.33126)) * i;
    /* We use a rounding fudge-factor of 0.5-epsilon for Cb and Cr.
     * This ensures that the maximum output will round to MAXJSAMPLE
     * not MAXJSAMPLE+1, and thus that we don't have to range-limit.
     */
    rgb_ycc_tab[i+B_CB_OFF] = FIX2(0.50000) * i    + CBCR_OFFSET + ONE_HALF-1;
/*  B=>Cb and R=>Cr tables are the same
    rgb_ycc_tab[i+R_CR_OFF] = FIX2(0.50000) * i    + CBCR_OFFSET + ONE_HALF-1;
*/
    rgb_ycc_tab[i+G_CR_OFF] = (-FIX2(0.41869)) * i;
    rgb_ycc_tab[i+B_CR_OFF] = (-FIX2(0.08131)) * i;
  }
}


/*
 * Convert some rows of samples to the JPEG colorspace.
 *
 * Note that we change from the application's interleaved-pixel format
 * to our internal noninterleaved, one-plane-per-component format.
 * The input buffer is therefore three times as wide as the output buffer.
 *
 * A starting row offset is provided only for the output buffer.  The caller
 * can easily adjust the passed input_buf value to accommodate any row
 * offset required on that side.
 */

void rgb_ycc_convert ()
{
 
  int r, g, b;
  INT32 * ctab = rgb_ycc_tab;
  JSAMPLE workspace[DCTSIZE2*3];
  DCTELEM output_buf0[DCTSIZE2],output_buf1[DCTSIZE2],output_buf2[DCTSIZE2];
  JSAMPROW inptr;
  JDIMENSION row,col;
  int cpt=0;
  
  rgb_ycc_start();
  while(cpt++<((IMAGE_WIDTH*IMAGE_HEIGHT)/DCTSIZE2)){
    Read_From_FIFO1(workspace);
    inptr = workspace;
    for(row=0;row<DCTSIZE;row++){
      for (col = 0; col < DCTSIZE; col++) {
	r = GETJSAMPLE(inptr[RGB_RED]);
	g = GETJSAMPLE(inptr[RGB_GREEN]);
	b = GETJSAMPLE(inptr[RGB_BLUE]);
	inptr += RGB_PIXELSIZE;
	
	/* Y */
	output_buf0[row*DCTSIZE+col] = (DCTELEM)
	  ((ctab[r+R_Y_OFF] + ctab[g+G_Y_OFF] + ctab[b+B_Y_OFF])
	   >> SCALEBITS);
	/* Cb */
	output_buf1[row*DCTSIZE+col] = (DCTELEM)
	  ((ctab[r+R_CB_OFF] + ctab[g+G_CB_OFF] + ctab[b+B_CB_OFF])
	   >> SCALEBITS);
	/* Cr */
	output_buf2[row*DCTSIZE+col] = (DCTELEM)
	  ((ctab[r+R_CR_OFF] + ctab[g+G_CR_OFF] + ctab[b+B_CR_OFF])
	   >> SCALEBITS);
      }
    }

    Write_In_FIFO2(output_buf0);
    Write_In_FIFO2(output_buf1);
    Write_In_FIFO2(output_buf2);

  }
  while(1){OSTimeDlyHMSM(0,0,0,100);};
}
